package flow.components

import chisel3.util.BitPat

object RV32Inst {
  private val bp = BitPat

  // format: off
  val lui    = this.bp("b???????_?????_?????_???_?????_01101_11")
  val auipc  = this.bp("b???????_?????_?????_???_?????_00101_11")

  val jal    = this.bp("b???????_?????_?????_???_?????_11011_11")
  val jalr   = this.bp("b???????_?????_?????_???_?????_11001_11")
  val beq    = this.bp("b???????_?????_?????_000_?????_11000_11")
  val bne    = this.bp("b???????_?????_?????_001_?????_11000_11")
  val blt    = this.bp("b???????_?????_?????_100_?????_11000_11")
  val bge    = this.bp("b???????_?????_?????_101_?????_11000_11")
  val bltu   = this.bp("b???????_?????_?????_110_?????_11000_11")
  val bgeu   = this.bp("b???????_?????_?????_111_?????_11000_11")

  val lb     = this.bp("b???????_?????_?????_000_?????_00000_11")
  val lh     = this.bp("b???????_?????_?????_001_?????_00000_11")
  val lw     = this.bp("b???????_?????_?????_010_?????_00000_11")
  val lbu    = this.bp("b???????_?????_?????_100_?????_00000_11")
  val lhu    = this.bp("b???????_?????_?????_101_?????_00000_11")
  val sb     = this.bp("b???????_?????_?????_000_?????_01000_11")
  val sh     = this.bp("b???????_?????_?????_001_?????_01000_11")
  val sw     = this.bp("b???????_?????_?????_010_?????_01000_11")

  val addi   = this.bp("b???????_?????_?????_000_?????_00100_11")
  val slti   = this.bp("b???????_?????_?????_010_?????_00100_11")
  val sltiu  = this.bp("b???????_?????_?????_011_?????_00100_11")
  val xori   = this.bp("b???????_?????_?????_100_?????_00100_11")
  val ori    = this.bp("b???????_?????_?????_110_?????_00100_11")
  val andi   = this.bp("b???????_?????_?????_111_?????_00100_11")
  val slli   = this.bp("b0000000_?????_?????_001_?????_00100_11")
  val srli   = this.bp("b0000000_?????_?????_101_?????_00100_11")
  val srai   = this.bp("b0100000_?????_?????_101_?????_00100_11")
  val add    = this.bp("b0000000_?????_?????_000_?????_01100_11")
  val sub    = this.bp("b0100000_?????_?????_000_?????_01100_11")
  val sll    = this.bp("b0000000_?????_?????_001_?????_01100_11")
  val slt    = this.bp("b0000000_?????_?????_010_?????_01100_11")
  val sltu   = this.bp("b0000000_?????_?????_011_?????_01100_11")
  val xor    = this.bp("b0000000_?????_?????_100_?????_01100_11")
  val srl    = this.bp("b0000000_?????_?????_101_?????_01100_11")
  val sra    = this.bp("b0100000_?????_?????_101_?????_01100_11")
  val or     = this.bp("b0000000_?????_?????_110_?????_01100_11")
  val and    = this.bp("b0000000_?????_?????_111_?????_01100_11")

  val ebreak = this.bp("b0000000_00001_00000_000_00000_11100_11")

  val csrrw  = this.bp("b???????_?????_?????_001_?????_11100_11")


  val mul    = this.bp("b0000001_?????_?????_000_?????_01100_11")
  val mulh   = this.bp("b0000001_?????_?????_001_?????_01100_11")
  val mulhsu = this.bp("b0000001_?????_?????_010_?????_01100_11")
  val mulhu  = this.bp("b0000001_?????_?????_011_?????_01100_11")
  val div    = this.bp("b0000001_?????_?????_100_?????_01100_11")
  val divu   = this.bp("b0000001_?????_?????_101_?????_01100_11")
  val rem    = this.bp("b0000001_?????_?????_110_?????_01100_11")
  val remu   = this.bp("b0000001_?????_?????_111_?????_01100_11")

  val inv    = this.bp("b???????_?????_?????_???_?????_?????_??")
  // format: on
}

import chisel3._
import chisel3.util._
import flow.Params
object RV32InstSubfields {
  implicit class ImmDecoder(val inst: UInt) extends AnyVal {
    def immB: UInt = Cat(
      Fill(20, inst(31)),
      inst(7),
      inst(30, 25),
      inst(11, 8),
      0.U(1.W)
    )

    def immI: UInt = Cat(Fill(20, inst(31)), inst(31, 20))

    def immJ: UInt = Cat(
      Fill(12, inst(31)),
      inst(19, 12),
      inst(20),
      inst(30, 25),
      inst(24, 21),
      0.U(1.W)
    )

    def immS: UInt = Cat(
      Fill(20, inst(31)),
      inst(31),
      inst(30, 25),
      inst(11, 8),
      inst(7)
    )

    def immU: UInt = Cat(inst(31, 12), 0.U(12.W))

    def rd: UInt = inst(11, 7)
    def rs1: UInt = inst(19, 15)
    def rs2: UInt = inst(24, 20)
  }
}
